@@ -467,7 +467,7 @@
             if(isTRUE(partialH))
             {
                 if(idlink0==2) pbH0[NPROB+NEF+NVC+NW+1:ntrtot0] <- 1
-                pbH0[posfix] <- 0
+                #pbH0[posfix] <- 0
                 if(sum(pbH0)==0 & Hr0==1) stop("No partial Hessian matrix can be defined")
             }
         }
@@ -475,10 +475,14 @@
         {
             if(!all(Hr0 %in% 1:NPM)) stop("Indexes in partialH are not correct")
             pbH0[Hr0] <- 1
-            pbH0[posfix] <- 0
+            #pbH0[posfix] <- 0
         }
         indexHr <- NULL
-        if(sum(pbH0)>0) indexHr <- which(pbH0==1)
+        if(sum(pbH0)>0)
+        {
+            if(length(posfix)) pbH1 <- pbH0[-posfix] else pbH1 <- pbH0
+            indexHr <- which(pbH1==1)
+        }
 
 
 
@@ -667,17 +671,17 @@
                                         up <- vbb[upper.tri(vbb,diag=TRUE)]
                                         vbb <- t(vbb)
                                         vbb[upper.tri(vbb,diag=TRUE)] <- up
-                                        Chol <- chol(vbb)
-                                        Chol <- t(Chol)
+                                        ##Chol <- chol(vbb)
+                                        ##Chol <- t(Chol)
 
                                         if(idg0[1]>1)
                                             {
-                                                b[c((NPROB+ng):(NPROB+NEF+NVC),(NPROB+NEF+NVC+NW+1):NPM)] <- bb + Chol %*% rnorm(length(bb))
+                                                b[c((NPROB+ng):(NPROB+NEF+NVC),(NPROB+NEF+NVC+NW+1):NPM)] <- rmvnorm(n=1,mean=bb,sigma = vbb) #bb + Chol %*% rnorm(length(bb))
                                                 b[NPROB+1:(ng-1)] <- 0
                                           } 
                                         else
                                             {
-                                                b[c((NPROB+1):(NPROB+NEF+NVC),(NPROB+NEF+NVC+NW+1):NPM)] <- bb + Chol %*% rnorm(length(bb))
+                                                b[c((NPROB+1):(NPROB+NEF+NVC),(NPROB+NEF+NVC+NW+1):NPM)] <- rmvnorm(n=1,mean=bb,sigma = vbb) #bb + Chol %*% rnorm(length(bb))
                                             }
 
                                         b[1:NPROB] <- 0
@@ -1055,9 +1059,50 @@
 ### ad 2/04/2012
         if (!("intercept" %in% nom.X0)) X0.names2 <- X0.names2[-1]
 ### ad
+
+        ## levels = modalites des variables dans X0 (si facteurs)
+        levelsdata <- vector("list", length(X0.names2))
+        levelsfixed <- vector("list", length(X0.names2))
+        levelsrandom <- vector("list", length(X0.names2))
+        levelsmixture <- vector("list", length(X0.names2))
+        levelsclassmb <- vector("list", length(X0.names2))
+        names(levelsdata) <- X0.names2
+        names(levelsfixed) <- X0.names2
+        names(levelsrandom) <- X0.names2
+        names(levelsmixture) <- X0.names2
+        names(levelsclassmb) <- X0.names2
+        for(v in X0.names2)
+        {
+            if(v == "intercept") next
+            
+            if(is.factor(data[,v]))
+            {
+                levelsdata[[v]] <- levels(data[,v])
+            }
+            if(length(grep(paste("factor\\(",v,"\\)",sep=""), fixed)))
+            {
+                levelsfixed[[v]] <- levels(as.factor(data[,v]))
+            }
+            if(length(grep(paste("factor\\(",v,"\\)",sep=""), random)))
+            {
+                levelsrandom[[v]] <- levels(as.factor(data[,v]))
+            }
+            if(length(grep(paste("factor\\(",v,"\\)",sep=""), mixture)))
+            {
+                levelsmixture[[v]] <- levels(as.factor(data[,v]))
+            }
+            if(length(grep(paste("factor\\(",v,"\\)",sep=""), classmb)))
+            {
+                levelsclassmb[[v]] <- levels(as.factor(data[,v]))
+            }
+            
+        }
+        levels <- list(levelsdata=levelsdata, levelsfixed=levelsfixed, levelsrandom=levelsrandom,
+                       levelsmixture=levelsmixture, levelsclassmb=levelsclassmb)
+
         cost <- proc.time()-ptm
         
-        res <-list(ns=ns0,ng=ng0,idea0=idea0,idprob0=idprob0,idg0=idg0,idcor0=idcor0,loglik=out$loglik,best=out$best,V=V,gconv=out$gconv,conv=out$conv,call=call,niter=out$niter,N=N,idiag=idiag0,pred=pred,pprob=ppi,predRE=predRE,Xnames=nom.X0,Xnames2=X0.names2,cholesky=Cholesky,estimlink=estimlink,epsY=epsY,linktype=idlink0,linknodes=zitr,Ydiscrete=Ydiscrete,discrete_loglik=out$vraisdiscret,UACV=out$UACV,IndivContrib=out$rlindiv,na.action=na.action,AIC=2*(length(out$best)-length(posfix)-out$loglik),BIC=(length(out$best)-length(posfix))*log(ns0)-2*out$loglik,data=datareturn,wRandom=wRandom,b0Random=b0Random, var.time=var.time, runtime=cost[3])
+        res <-list(ns=ns0,ng=ng0,idea0=idea0,idprob0=idprob0,idg0=idg0,idcor0=idcor0,loglik=out$loglik,best=out$best,V=V,gconv=out$gconv,conv=out$conv,call=call,niter=out$niter,N=N,idiag=idiag0,pred=pred,pprob=ppi,predRE=predRE,Xnames=nom.X0,Xnames2=X0.names2,cholesky=Cholesky,estimlink=estimlink,epsY=epsY,linktype=idlink0,linknodes=zitr,Ydiscrete=Ydiscrete,discrete_loglik=out$vraisdiscret,UACV=out$UACV,IndivContrib=out$rlindiv,na.action=na.action,AIC=2*(length(out$best)-length(posfix)-out$loglik),BIC=(length(out$best)-length(posfix))*log(ns0)-2*out$loglik,data=datareturn,wRandom=wRandom,b0Random=b0Random, levels=levels, var.time=var.time, runtime=cost[3])
         class(res) <- c("lcmm") 
 
         if(verbose==TRUE) cat("The program took", round(cost[3],2), "seconds \n") 
