@@ -231,74 +231,73 @@ epoce <- function(model,pred.times,var.time,fun.time=identity,newdata=NULL,subse
 
     
 ### pour les facteurs
-
-    ##cas ou une variable du dataset est un facteur
-    olddata <- eval(model$call$data)
+    ## transform to factor is the variable appears in levels$levelsdata
     for(v in colnames(data))
+    {
+        if(v %in% names(model$levels$levelsdata))
         {
-            if (is.factor(olddata[,v]) & !(is.factor(data[,v])))
-                {
-                    mod <- levels(olddata[,v])
-                    if (!(levels(as.factor(data[,v])) %in% mod)) stop(paste("invalid level in factor", v))
-                    data[,v] <- factor(data[,v], levels=mod)
-                }
+            if(!is.null(model$levels$levelsdata[[v]]))
+            {
+                data[,v] <- factor(data[,v], levels=model$levels$levelsdata[[v]])
+            }
         }
+    }
     
-    ##cas ou on a factor() dans l'appel
-    z <- all.names(call_fixed)
-    ind_factor <- which(z=="factor")
-    if(length(ind_factor))
-        {
-            nom.factor <- z[ind_factor+1]  
-            for (v in nom.factor)
-                {
-                    mod <- levels(as.factor(olddata[,v]))
-                    if (!all(levels(as.factor(data[,v])) %in% mod)) stop(paste("invalid level in factor", v))
-                    data[,v] <- factor(data[,v], levels=mod)
-                }
-        }
+    ## ##cas ou on a factor() dans l'appel
+    ## z <- all.names(call_fixed)
+    ## ind_factor <- which(z=="factor")
+    ## if(length(ind_factor))
+    ##     {
+    ##         nom.factor <- z[ind_factor+1]  
+    ##         for (v in nom.factor)
+    ##             {
+    ##                 mod <- levels(as.factor(olddata[,v]))
+    ##                 if (!all(levels(as.factor(data[,v])) %in% mod)) stop(paste("invalid level in factor", v))
+    ##                 data[,v] <- factor(data[,v], levels=mod)
+    ##             }
+    ##     }
     call_fixed <- gsub("factor","",call_fixed)
 
-    z <- all.names(call_random)
-    ind_factor <- which(z=="factor")
-    if(length(ind_factor))
-        {
-            nom.factor <- z[ind_factor+1]
-            for (v in nom.factor)
-                {
-                    mod <- levels(as.factor(olddata[,v]))
-                    if (!all(levels(as.factor(data[,v])) %in% mod)) stop(paste("invalid level in factor", v))
-                    data[,v] <- factor(data[,v], levels=mod)
-                }
-        }
+    ## z <- all.names(call_random)
+    ## ind_factor <- which(z=="factor")
+    ## if(length(ind_factor))
+    ##     {
+    ##         nom.factor <- z[ind_factor+1]
+    ##         for (v in nom.factor)
+    ##             {
+    ##                 mod <- levels(as.factor(olddata[,v]))
+    ##                 if (!all(levels(as.factor(data[,v])) %in% mod)) stop(paste("invalid level in factor", v))
+    ##                 data[,v] <- factor(data[,v], levels=mod)
+    ##             }
+    ##     }
     call_random <- gsub("factor","",call_random)
     
-    z <- all.names(call_classmb)
-    ind_factor <- which(z=="factor")
-    if(length(ind_factor))
-        {
-            nom.factor <- z[ind_factor+1]
-            for (v in nom.factor)
-                {
-                    mod <- levels(as.factor(olddata[,v]))
-                    if (!all(levels(as.factor(data[,v])) %in% mod)) stop(paste("invalid level in factor", v))
-                    data[,v] <- factor(data[,v], levels=mod)
-                }
-        }
+    ## z <- all.names(call_classmb)
+    ## ind_factor <- which(z=="factor")
+    ## if(length(ind_factor))
+    ##     {
+    ##         nom.factor <- z[ind_factor+1]
+    ##         for (v in nom.factor)
+    ##             {
+    ##                 mod <- levels(as.factor(olddata[,v]))
+    ##                 if (!all(levels(as.factor(data[,v])) %in% mod)) stop(paste("invalid level in factor", v))
+    ##                 data[,v] <- factor(data[,v], levels=mod)
+    ##             }
+    ##     }
     call_classmb <- gsub("factor","",call_classmb)
     
-    z <- all.names(call_survival)
-    ind_factor <- which(z=="factor")
-    if(length(ind_factor))
-        {
-            nom.factor <- z[ind_factor+1]
-            for (v in nom.factor)
-                {
-                    mod <- levels(as.factor(olddata[,v]))
-                    if (!all(levels(as.factor(data[,v])) %in% mod)) stop(paste("invalid level in factor", v))
-                    data[,v] <- factor(data[,v], levels=mod)
-                }
-        }
+    ## z <- all.names(call_survival)
+    ## ind_factor <- which(z=="factor")
+    ## if(length(ind_factor))
+    ##     {
+    ##         nom.factor <- z[ind_factor+1]
+    ##         for (v in nom.factor)
+    ##             {
+    ##                 mod <- levels(as.factor(olddata[,v]))
+    ##                 if (!all(levels(as.factor(data[,v])) %in% mod)) stop(paste("invalid level in factor", v))
+    ##                 data[,v] <- factor(data[,v], levels=mod)
+    ##             }
+    ##     }
     
     call_survival <- gsub("factor","",call_survival)
 
@@ -448,22 +447,74 @@ epoce <- function(model,pred.times,var.time,fun.time=identity,newdata=NULL,subse
         }
 
 
+    ## create one data frame for each formula (useful with factors)
+    newdata1fixed <- newdata1
+    for(v in colnames(newdata1fixed))
+    {
+        if(v %in% names(model$levels$levelsfixed))
+        {
+            if(!is.null(model$levels$levelsfixed[[v]]))
+            {
+                newdata1fixed[,v] <- factor(newdata1fixed[,v], levels=model$levels$levelsfixed[[v]])
+                if(any(is.na(newdata1fixed[,v]))) stop(paste("Wrong factor level in variable",v))
+            }
+        }
+    }
+    newdata1random <- newdata1
+    for(v in colnames(newdata1random))
+    {
+        if(v %in% names(model$levels$levelsrandom))
+        {
+            if(!is.null(model$levels$levelsrandom[[v]]))
+            {
+                newdata1random[,v] <- factor(newdata1random[,v], levels=model$levels$levelsrandom[[v]])
+                if(any(is.na(newdata1random[,v]))) stop(paste("Wrong factor level in variable",v))
+            }
+        }
+    }
+    newdata1classmb <- newdata1
+    for(v in colnames(newdata1classmb))
+    {
+        if(v %in% names(model$levels$levelsclassmb))
+        {
+            if(!is.null(model$levels$levelsclassmb[[v]]))
+            {
+                newdata1classmb[,v] <- factor(newdata1classmb[,v], levels=model$levels$levelsclassmb[[v]])
+                if(any(is.na(newdata1classmb[,v]))) stop(paste("Wrong factor level in variable",v))
+            }
+        }
+    }
+    newdata1surv <- newdata1
+    for(v in colnames(newdata1surv))
+    {
+        if(v %in% names(model$levels$levelssurv))
+        {
+            if(!is.null(model$levels$levelssurv[[v]]))
+            {
+                newdata1surv[,v] <- factor(newdata1surv[,v], levels=model$levels$levelssurv[[v]])
+                if(any(is.na(newdata1surv[,v]))) stop(paste("Wrong factor level in variable",v))
+            }
+        }
+    }
+    
+    
 ###creation de X0 (ttes les var + interactions)
 
     X_intercept <- model.matrix(~1,data=newdata1)
+    colnames(X_intercept) <- "intercept"
     
     ## fixed
 
-    X_fixed <- model.matrix(formula(paste("~",call_fixed,sep="")),data=newdata1)
+    X_fixed <- model.matrix(formula(paste("~",call_fixed,sep="")),data=newdata1fixed)
     if(colnames(X_fixed)[1]=="(Intercept)")
         {
-            colnames(X_fixed)[1] <- "intercept"
+            X_fixed <- X_fixed[,-1,drop=FALSE]        
         }
 
     ## random
     if(!is.null(model$call$random))
         {
-            X_random <- model.matrix(formula(paste("~",call_random,sep="")),data=newdata1)
+            X_random <- model.matrix(formula(paste("~",call_random,sep="")),data=newdata1random)
             if(colnames(X_random)[1]=="(Intercept)")
                 {
                     colnames(X_random)[1] <- "intercept"
@@ -477,7 +528,7 @@ epoce <- function(model,pred.times,var.time,fun.time=identity,newdata=NULL,subse
     ## classmb
     if(!is.null(model$call$classmb))
         {
-            X_classmb <- model.matrix(formula(paste("~",call_classmb,sep="")),data=newdata1)
+            X_classmb <- model.matrix(formula(paste("~",call_classmb,sep="")),data=newdata1classmb)
             if(colnames(X_classmb)[1]=="(Intercept)")
                 {
                     colnames(X_classmb)[1] <- "intercept"
@@ -491,7 +542,7 @@ epoce <- function(model,pred.times,var.time,fun.time=identity,newdata=NULL,subse
     ## survival
     if(!is.null(model$call$survival))
         {
-            X_survival <- model.matrix(formula(paste("~",call_survival,sep="")),data=newdata1)
+            X_survival <- model.matrix(formula(paste("~",call_survival,sep="")),data=newdata1surv)
             if(colnames(X_survival)[1]=="(Intercept)")
                 {
                     colnames(X_survival)[1] <- "intercept"
@@ -515,18 +566,69 @@ epoce <- function(model,pred.times,var.time,fun.time=identity,newdata=NULL,subse
     
     
     ## Construction de X0 dans le bon ordre
-    X <- cbind(X_intercept,X_fixed,X_random,X_classmb,X_survival,X_cor)
-    colX <- strsplit(colnames(X),split=":",fixed=TRUE)
-    colX <- lapply(colX,sort)
-    colX <- lapply(colX,paste,collapse=":")
-    colnames(X) <- unlist(colX)
+    X0 <- cbind(X_intercept,X_fixed)
+    colX <- c("intercept",colnames(X_fixed))
+    if(!is.null(X_random))
+    {
+        for(i in 1:length(colnames(X_random)))
+        {
+            if((colnames(X_random)[i] %in% colnames(X0))==FALSE)
+            {
+                X0 <- cbind(X0,X_random[,i])
+                colnames(X0) <- c(colX,colnames(X_random)[i])
+                colX <- colnames(X0)
+            }	 
+        }
+    }
+    if(!is.null(X_classmb))
+    {
+        for(i in 1:length(colnames(X_classmb)))
+        {
+            if((colnames(X_classmb)[i] %in% colnames(X0))==FALSE)
+            {
+                X0 <- cbind(X0,X_classmb[,i])
+                colnames(X0) <- c(colX,colnames(X_classmb)[i])
+                colX <- colnames(X0)
+            }	
+        }
+    }
+    if(!is.null(X_survival))
+    {
+        for(i in 1:length(colnames(X_survival)))
+        {
+            if((colnames(X_survival)[i] %in% colnames(X0))==FALSE)
+            {
+                X0 <- cbind(X0,X_survival[,i])
+                colnames(X0) <- c(colX,colnames(X_survival)[i])
+                colX <- colnames(X0)
+            }
+        }
+    }
     
-    Xnames <- strsplit(model$Names$Xnames,split=":",fixed=TRUE)
-    Xnames <- lapply(Xnames,sort)
-    Xnames <- lapply(Xnames,paste,collapse=":")
-    Xnames <- unlist(Xnames)
+    if(model$N[7]>0)
+    {
+        idspecif <- matrix(model$idspecif,nbevt,length(model$idg),byrow=TRUE)
+        if(model$idea[z]==0 & model$idprob[z]==0 & model$idg[z]==0 & model$idcom[z]==0 & all(idspecif[,z]==0))
+        {
+            X0 <- cbind(X0,X_cor)
+            colnames(X0) <- c(colX,model$Names$Xnames[z])
+            colX <- colnames(X0)
+        }
+        
+    }
     
-    X0 <- X[,Xnames]
+    ## X <- cbind(X_intercept,X_fixed,X_random,X_classmb,X_survival,X_cor)
+    ## colX <- strsplit(colnames(X),split=":",fixed=TRUE)
+    ## colX <- lapply(colX,sort)
+    ## colX <- lapply(colX,paste,collapse=":")
+    ## colnames(X) <- unlist(colX)
+    
+    ## Xnames <- strsplit(model$Names$Xnames,split=":",fixed=TRUE)
+    ## Xnames <- lapply(Xnames,sort)
+    ## Xnames <- lapply(Xnames,paste,collapse=":")
+    ## Xnames <- unlist(Xnames)
+    
+    ## X0 <- X[,Xnames]
 ###X0 fini  
 
 
