@@ -1,4 +1,4 @@
-
+#' @export
 estimates.lcmm <- function(x,cholesky=TRUE)
 {
  if(missing(x)) stop("The argument x should be specified")
@@ -33,5 +33,3 @@ estimates.lcmm <- function(x,cholesky=TRUE)
  
  return(res)
 }
-
-estimates <- function(x,cholesky=TRUE) UseMethod("estimates")
