@@ -294,7 +294,7 @@
 #' 
 #' 
 hlme <-
-    function(fixed,mixture,random,subject,classmb,ng=1,idiag=FALSE,nwg=FALSE,cor=NULL,data,B,convB=0.0001,convL=0.0001,convG=0.0001,prior,pprior=NULL,maxiter=500,subset=NULL,na.action=1,posfix=NULL,verbose=TRUE,returndata=FALSE,var.time=NULL,partialH=FALSE,nproc=1,clustertype=NULL){
+    function(fixed,mixture,random,subject,classmb,ng=1,idiag=FALSE,nwg=FALSE,cor=NULL,data,B,convB=0.0001,convL=0.0001,convG=0.0001,prior,pprior=NULL,maxiter=500,subset=NULL,na.action=1,posfix=NULL,verbose=FALSE,returndata=FALSE,var.time=NULL,partialH=FALSE,nproc=1,clustertype=NULL){
 
         ptm<-proc.time()
 
@@ -677,12 +677,12 @@ hlme <-
         ##pprior : proba d'appartenance a chaque classe
         if(is.null(pprior))
         {
-            pprior0 <- matrix(0, length(IND), ng)            
+            pprior0 <- matrix(1, length(IND), ng)            
         }
         else
         {
             if(ng==1) stop("pprior is only useful if ng>1")
-            if(classmb != ~-1) stop("classmb should be ~-1 if pprior is specified")
+            ##if(classmb != ~-1) stop("classmb should be ~-1 if pprior is specified")
             if(!is.character(pprior)) stop("pprior should be a character vector")
             if(length(pprior) != ng) stop("pprior should be of length ng")
             if(!all(pprior %in% colnames(newdata))) stop("pprior variables should be included in data")
@@ -767,13 +767,13 @@ hlme <-
         {
             pprior0 <- matYX[cumsum(nmes0),3+1:ng, drop=FALSE]
             if(any(is.na(pprior0))) stop("pprior contains missing values")
-            if(any(pprior0 < 0)) stop("pprior should be non negative")
-            if(any(pprior0 > 1)) stop("pprior should be < 1")
-            if(!all.equal(as.numeric(apply(pprior0,1,sum)), rep(1,ns0))) stop("pprior should sum up to 1 for all subjects")
+            #if(any(pprior0 < 0)) stop("pprior should be non negative")
+            #if(any(pprior0 > 1)) stop("pprior should be < 1")
+            #if(!all.equal(as.numeric(apply(pprior0,1,sum)), rep(1,ns0))) stop("pprior should sum up to 1 for all subjects")
         }
         else
         {
-            pprior0 <- matrix(0, ns0, ng0)
+            pprior0 <- matrix(1, ns0, ng0)
         }
         pprior0 <- t(pprior0)       
         
@@ -792,6 +792,7 @@ hlme <-
         pred_ss_g <- rep(0,nobs0*ng0)
         nea0 <- sum(idea0==1)
         predRE <- rep(0,nea0*ns0)
+        varRE <- rep(0,nea0*(nea0+1)/2*ns0)
 
         ##---------------------------------------------------------------------------
         ##definition du vecteur de parametres + initialisation
@@ -807,7 +808,7 @@ hlme <-
                 
                 Brandom <- TRUE
                 B <- eval(cl$B[[2]])
-
+                if(B$conv != 1) stop("Model in argument B did not converge properly")
                 #if(length(posfix)) stop("Argument posfix is not compatible with random intial values")
             }
 
@@ -880,17 +881,21 @@ hlme <-
         if(is.logical(partialH))
         {
             if(partialH) pbH0 <- rep(1,NPM)
-            pbH0[posfix] <- 0
+            #pbH0[posfix] <- 0
             if(sum(pbH0)==0 & Hr0==1) stop("No partial Hessian matrix can be defined")
         }
         else
         {
             if(!all(Hr0 %in% 1:NPM)) stop("Indexes in partialH are not correct")
             pbH0[Hr0] <- 1
-            pbH0[posfix] <- 0
+            #pbH0[posfix] <- 0
         }
         indexHr <- NULL
-        if(sum(pbH0)>0) indexHr <- which(pbH0==1)
+        if(sum(pbH0)>0)
+        {
+            if(length(posfix)) pbH1 <- pbH0[-posfix] else pbH1 <- pbH0
+            indexHr <- which(pbH1==1)
+        }
         
         if(missing(B)){
 
@@ -1049,10 +1054,10 @@ hlme <-
                                         up <- vbb[upper.tri(vbb,diag=TRUE)]
                                         vbb <- t(vbb)
                                         vbb[upper.tri(vbb,diag=TRUE)] <- up
-                                        Chol <- chol(vbb)
-                                        Chol <- t(Chol)
+                                        ##Chol <- chol(vbb)
+                                        ##Chol <- t(Chol)
                                         
-                                        b[c((NPROB+1):(NPROB+NEF+NVC),(NPROB+NEF+NVC+NW+1):NPM)] <- bb + Chol %*% rnorm(NPM-NPROB-NW)
+                                        b[c((NPROB+1):(NPROB+NEF+NVC),(NPROB+NEF+NVC+NW+1):NPM)] <- rmvnorm(n=1,mean=bb,sigma = vbb) #bb + Chol %*% rnorm(NPM-NPROB-NW)
 
                                         if(NPROB>0) b[1:NPROB] <- 0
                                         if(NW>0) b[NPROB+NEF+NVC+1:NW] <- 1
@@ -1182,6 +1187,7 @@ hlme <-
             
             out <- list(conv=2, V=rep(NA, NPM*(NPM+1)/2), best=b,
                         ppi2=rep(NA,ns0*ng0), predRE=rep(NA,ns0*nea0),
+                        varRE=rep(NA,ns0*nea0*(nea0+1)/2),
                         resid_m=rep(NA,nobs0), resid_ss=rep(NA,nobs0),
                         pred_m_g=rep(NA,nobs0*ng0), pred_ss_g=rep(NA,nobs0*ng0),
                         gconv=rep(NA,3), niter=0, loglik=vrais)
@@ -1201,6 +1207,7 @@ hlme <-
             
             out <- list(conv=res$istop, V=res$v, best=res$b,
                         ppi2=rep(NA,ns0*ng0), predRE=rep(NA,ns0*nea0),
+                        varRE=rep(NA,ns0*nea0*(nea0+1)/2),
                         resid_m=rep(NA,nobs0), resid_ss=rep(NA,nobs0),
                         pred_m_g=rep(NA,nobs0*ng0), pred_ss_g=rep(NA,nobs0*ng0),
                         gconv=c(res$ca, res$cb, res$rdm), niter=res$ni,
@@ -1217,6 +1224,7 @@ hlme <-
             pred_m_g <- rep(0,nobs0*ng0)
             pred_ss_g <- rep(0,nobs0*ng0)
             predRE <- rep(0,ns0*nea0)
+            varRE <- rep(0,ns0*nea0*(nea0+1)/2)
             
             post <- .Fortran(C_loglikhlme,
                              as.double(Y0),
@@ -1244,6 +1252,7 @@ hlme <-
                              pred_m_g=as.double(pred_m_g),
                              pred_ss_g=as.double(pred_ss_g),
                              predRE=as.double(predRE),
+                             varRE=as.double(varRE),
                              as.integer(fix0),
                              as.integer(nfix),
                              as.double(bfix),
@@ -1252,6 +1261,7 @@ hlme <-
             
             out$ppi2 <- post$ppi
             out$predRE <- post$predRE
+            out$varRE <- post$varRE
             out$resid_m <- post$resid_m
             out$resid_ss <- post$resid_ss
             out$pred_m_g <- post$pred_m_g
@@ -1443,7 +1453,7 @@ hlme <-
         
         cost<-proc.time()-ptm
         
-        res <-list(ns=ns0,ng=ng0,idea0=idea0,idprob0=idprob0,idg0=idg0,idcor0=idcor0,loglik=out$loglik,best=out$best,V=V,gconv=out$gconv,conv=out$conv,call=cl,niter=out$niter,N=N,idiag=idiag0,pred=pred,pprob=ppi,predRE=predRE,Xnames=nom.X0,Xnames2=X0.names2,cholesky=Cholesky,na.action=na.action,AIC=2*(length(out$best)-length(posfix)-out$loglik),BIC=(length(out$best)-length(posfix))*log(ns0)-2*out$loglik,data=datareturn,wRandom=wRandom,b0Random=b0Random, levels=levels, var.time=var.time, runtime=cost[3])
+        res <-list(ns=ns0,ng=ng0,idea0=idea0,idprob0=idprob0,idg0=idg0,idcor0=idcor0,loglik=out$loglik,best=out$best,V=V,gconv=out$gconv,conv=out$conv,call=cl,niter=out$niter,N=N,idiag=idiag0,pred=pred,pprob=ppi,predRE=predRE,varRE=out$varRE,Xnames=nom.X0,Xnames2=X0.names2,cholesky=Cholesky,na.action=na.action,AIC=2*(length(out$best)-length(posfix)-out$loglik),BIC=(length(out$best)-length(posfix))*log(ns0)-2*out$loglik,data=datareturn,wRandom=wRandom,b0Random=b0Random, levels=levels, var.time=var.time, runtime=cost[3])
         class(res) <-c("hlme") 
 
         if(verbose==TRUE) cat("The program took", round(cost[3],2), "seconds \n")
